@@ -1,4 +1,5 @@
 #include "imager.h"
+#include "imageri.h"
 #include <stdlib.h>
 #ifndef _MSC_VER
 #include <unistd.h>
@@ -190,6 +191,9 @@ myfree_file_line(void *p, char *file, int line) {
   char  *pp = p;
   int match = 0;
   int i;
+
+  if (p == NULL)
+    return;
   
   for(i=0; i<MAXMAL; i++) if (malloc_pointers[i].ptr == p) {
     mm_log((1,"myfree_file_line: pointer %i (%s) freed at %s (%i)\n", i, malloc_pointers[i].comm, file, line));
@@ -317,13 +321,13 @@ i_mempool_destroy(i_mempool *mp) {
 #undef min
 #undef max
 
-int
-i_min(int a,int b) {
+i_img_dim
+i_minx(i_img_dim a, i_img_dim b) {
   if (a<b) return a; else return b;
 }
 
-int
-i_max(int a,int b) {
+i_img_dim
+i_maxx(i_img_dim a, i_img_dim b) {
   if (a>b) return a; else return b;
 }
 
@@ -342,19 +346,23 @@ struct utf8_size utf8_sizes[] =
 };
 
 /*
-=item utf8_advance(char **p, int *len)
+=item i_utf8_advance(char **p, size_t *len)
 
-Retreive a UTF8 character from the stream.
+Retrieve a C<UTF-8> character from the stream.
 
 Modifies *p and *len to indicate the consumed characters.
 
-This doesn't support the extended UTF8 encoding used by later versions
-of Perl.
+This doesn't support the extended C<UTF-8> encoding used by later
+versions of Perl.
+
+This doesn't check that the C<UTF-8> character is using the shortest
+possible representation.
 
 =cut
 */
 
-unsigned long i_utf8_advance(char const **p, int *len) {
+unsigned long 
+i_utf8_advance(char const **p, size_t *len) {
   unsigned char c;
   int i, ci, clen = 0;
   unsigned char codes[3];
@@ -365,6 +373,7 @@ unsigned long i_utf8_advance(char const **p, int *len) {
   for (i = 0; i < sizeof(utf8_sizes)/sizeof(*utf8_sizes); ++i) {
     if ((c & utf8_sizes[i].mask) == utf8_sizes[i].expect) {
       clen = utf8_sizes[i].size;
+      break;
     }
   }
   if (clen == 0 || *len < clen-1) {
